#include "PlotProbe.h"
#include "Vector.h"
#include <rfftw.h>
#include <math.h>

class SpectrumProbe;

DECLARE_NODE(SpectrumProbe)
/*Node

 * @name SpectrumProbe
 * @category Probe
 * @description No description available

 * @input_name INPUT
 * @input_description No description available

 * @output_name OUTPUT
 * @output_description No description available

 * @parameter_name BREAK_AT
 * @parameter_description No description available

 * @parameter_name SHOW
 * @parameter_description No description available

 * @parameter_name SKIP
 * @parameter_description No description available

 * @parameter_name LENGTH
 * @parameter_description No description available

 * @parameter_name SQUARE
 * @parameter_description No description available

 * @parameter_name LOG
 * @parameter_description No description available

END*/

class SpectrumProbe : public PlotProbe {
  protected:
      fftw_plan plan;
      int length;
      int psLength;
      float *window;
      bool calcLog, calcSquare;
  public:

   SpectrumProbe(string nodeName, ParameterSet params)
      : PlotProbe(nodeName, params)
      , calcSquare(false)
      , calcLog(false)
      {
	 length = dereference_cast <int> (parameters.get("LENGTH"));
	 psLength = length / 2;
	 plan = rfftw_create_plan (length, FFTW_FORWARD, FFTW_ESTIMATE);
	 window = new float [length];
	 for (int i=0;i<length;i++)
	    window[i] = .54-.46*cos(2*M_PI*i/length);
	 if (parameters.exist("SQUARE"))
	    calcSquare = dereference_cast<bool> (parameters.get("SQUARE"));
	 if (parameters.exist("LOG"))
	    calcLog = dereference_cast<bool> (parameters.get("LOG"));
	 
      }

   virtual ~SpectrumProbe() 
      {
	 rfftw_destroy_plan(plan);
	 delete [] window;
      }

   virtual void display()
      {
	 GnomeCanvasPoints *points;
	 
	 gdk_threads_enter();
	 
	 //cerr << "length = " << length << endl;
	 
	 float in[length];
	 float ps[length];
	 Vector<float> &data = object_cast<Vector<float> > (inputValue);
	 for (int i=0;i<length;i++)
	 {
	    in[i]=data[i]*window[i];
	 }

	 rfftw_one (plan, in, ps);
	 
	 points = gnome_canvas_points_new(psLength);
	 
	 ps[0]=ps[0]*ps[0];
	 for (int i=1;i<psLength;i++)
	 {
	    ps[i]=ps[i]*ps[i] + ps[length-i]*ps[length-i];
	 }

	 if (!calcSquare)
	    for (int i=0;i<psLength;i++)
	       ps[i]=sqrt(ps[i]);
	 
	 if (calcLog)
	    for (int i=0;i<psLength;i++)
	       ps[i]=log(ps[i]+FLT_MIN);

	 double datamin, datamax;
	 datamin = datamax = ps[0];
	 for (int i=0;i<psLength;i++)
	 {
	    if (ps[i] > datamax)
	       datamax = ps[i];
	    if (ps[i] < datamin)
	       datamin = ps[i];
	 }
	 datamax+=.00001;
	 for (int i=0;i<psLength;i++)
	 {
	    //points->coords[2*i]=(100.0*i)/length;
	    points->coords[2*i]=xmin+((xmax-xmin)*i)/(psLength-1);
	    points->coords[2*i+1]= ymin + (ymax-ymin)*(1-(ps[i]-datamin)/(datamax-datamin));
	 }
	 
	 gnome_canvas_item_set(item, "points", points, NULL);
	 
	 gnome_canvas_points_unref(points);
	 
	 //cerr << "plot done...\n";
	 
	 gdk_threads_leave();
      }


};

